@@ -1,11 +1,11 @@
 /* libdvdcss.c: DVD reading library.
  *
- * Authors: Stphane Borel <stef@via.ecp.fr>
- *          Samuel Hocevar <sam@zoy.org>
- *          Hkan Hjort <d95hjort@dtek.chalmers.se>
+ * Authors: Stéphane Borel <stef@via.ecp.fr>
+ *          Sam Hocevar <sam@zoy.org>
+ *          Håkan Hjort <d95hjort@dtek.chalmers.se>
  *
- * Copyright (C) 1998-2002 VideoLAN
- * $Id: libdvdcss.c,v 1.29 2003/01/29 22:59:35 massiot Exp $
+ * Copyright (C) 1998-2008 VideoLAN
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,7 +87,10 @@
  *     values. This will speed up descrambling of DVDs which are in the
  *     cache. The DVDCSS_CACHE directory is created if it does not exist,
  *     and a subdirectory is created named after the DVD's title or
- *     manufacturing date.
+ *     manufacturing date. If DVDCSS_CACHE is not set or is empty, \e libdvdcss
+ *     will use the default value which is "${HOME}/.dvdcss/" under Unix and
+ *     "C:\Documents and Settings\$USER\Application Data\dvdcss\" under Win32.
+ *     The special value "off" disables caching.
  */
 
 /*
@@ -103,6 +106,9 @@
 #ifdef HAVE_SYS_PARAM_H
 #   include <sys/param.h>
 #endif
+#ifdef HAVE_PWD_H
+#   include <pwd.h>
+#endif
 #include <fcntl.h>
 #include <errno.h>
 
@@ -150,10 +156,11 @@ char * dvdcss_interface_2 = VERSION;
  * and when possible, the disc key is retrieved.
  *
  * dvdcss_open() returns a handle to be used for all subsequent \e libdvdcss
- * calls. If an error occured, NULL is returned.
+ * calls. If an error occurred, NULL is returned.
  */
-extern dvdcss_t dvdcss_open ( char *psz_target )
+LIBDVDCSS_EXPORT dvdcss_t dvdcss_open ( char *psz_target )
 {
+    char psz_buffer[PATH_MAX];
     int i_ret;
 
     char *psz_method = getenv( "DVDCSS_METHOD" );
@@ -193,15 +200,10 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
      */
     if( psz_verbose != NULL )
     {
-        switch( atoi( psz_verbose ) )
-        {
-        case 2:
-            dvdcss->b_debug = 1;
-        case 1:
-            dvdcss->b_errors = 1;
-        case 0:
-            break;
-        }
+        int i = atoi( psz_verbose );
+
+        if( i >= 2 ) dvdcss->b_debug = i;
+        if( i >= 1 ) dvdcss->b_errors = 1;
     }
 
     /*
@@ -223,27 +225,108 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
         }
         else
         {
-            _dvdcss_error( dvdcss, "unknown decrypt method, please choose "
-                                   "from 'title', 'key' or 'disc'" );
+            print_error( dvdcss, "unknown decrypt method, please choose "
+                                 "from 'title', 'key' or 'disc'" );
             free( dvdcss->psz_device );
             free( dvdcss );
             return NULL;
         }
     }
 
+    /*
+     *  If DVDCSS_CACHE was not set, try to guess a default value
+     */
+    if( psz_cache == NULL || psz_cache[0] == '\0' )
+    {
+#ifdef HAVE_DIRECT_H
+        typedef HRESULT( WINAPI *SHGETFOLDERPATH )
+                       ( HWND, int, HANDLE, DWORD, LPTSTR );
+
+#   define CSIDL_FLAG_CREATE 0x8000
+#   define CSIDL_APPDATA 0x1A
+#   define SHGFP_TYPE_CURRENT 0
+
+        char psz_home[MAX_PATH];
+        HINSTANCE p_dll;
+        SHGETFOLDERPATH p_getpath;
+
+        *psz_home = '\0';
+
+        /* Load the shfolder dll to retrieve SHGetFolderPath */
+        p_dll = LoadLibrary( "shfolder.dll" );
+        if( p_dll )
+        {
+            p_getpath = (void*)GetProcAddress( p_dll, "SHGetFolderPathA" );
+            if( p_getpath )
+            {
+                /* Get the "Application Data" folder for the current user */
+                if( p_getpath( NULL, CSIDL_APPDATA | CSIDL_FLAG_CREATE,
+                               NULL, SHGFP_TYPE_CURRENT, psz_home ) == S_OK )
+                {
+                    FreeLibrary( p_dll );
+                }
+                else
+                {
+                    *psz_home = '\0';
+                }
+            }
+            FreeLibrary( p_dll );
+        }
+
+        /* Cache our keys in
+         * C:\Documents and Settings\$USER\Application Data\dvdcss\ */
+        if( *psz_home )
+        {
+            snprintf( psz_buffer, PATH_MAX, "%s/dvdcss", psz_home );
+            psz_buffer[PATH_MAX-1] = '\0';
+            psz_cache = psz_buffer;
+        }
+#else
+        char *psz_home = NULL;
+#   ifdef HAVE_PWD_H
+        struct passwd *p_pwd;
+
+        /* Try looking in password file for home dir. */
+        p_pwd = getpwuid(getuid());
+        if( p_pwd )
+        {
+            psz_home = p_pwd->pw_dir;
+        }
+#   endif
+
+        if( psz_home == NULL )
+        {
+            psz_home = getenv( "HOME" );
+        }
+        if( psz_home == NULL )
+        {
+            psz_home = getenv( "USERPROFILE" );
+        }
+
+        /* Cache our keys in ${HOME}/.dvdcss/ */
+        if( psz_home )
+        {
+            snprintf( psz_buffer, PATH_MAX, "%s/.dvdcss", psz_home );
+            psz_buffer[PATH_MAX-1] = '\0';
+            psz_cache = psz_buffer;
+        }
+#endif
+    }
+
     /*
      *  Find cache dir from the DVDCSS_CACHE environment variable
      */
     if( psz_cache != NULL )
     {
-        if( psz_cache[0] == '\0' )
+        if( psz_cache[0] == '\0' || !strcmp( psz_cache, "off" ) )
         {
             psz_cache = NULL;
         }
         /* Check that we can add the ID directory and the block filename */
-        else if( strlen( psz_cache ) + 1 + 32 + 1 + 10 + 1 > PATH_MAX )
+        else if( strlen( psz_cache ) + 1 + 32 + 1 + (KEY_SIZE * 2) + 10 + 1
+                  > PATH_MAX )
         {
-            _dvdcss_error( dvdcss, "cache directory name is too long" );
+            print_error( dvdcss, "cache directory name is too long" );
             psz_cache = NULL;
         }
     }
@@ -251,6 +334,7 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
     /*
      *  Open device
      */
+    _dvdcss_check( dvdcss );
     i_ret = _dvdcss_open( dvdcss );
     if( i_ret < 0 )
     {
@@ -268,14 +352,14 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
         if( i_ret < 0 )
         {
             /* Disable the CSS ioctls and hope that it works? */
-            _dvdcss_debug( dvdcss,
-                           "could not check whether the disc was scrambled" );
+            print_debug( dvdcss,
+                         "could not check whether the disc was scrambled" );
             dvdcss->b_ioctls = 0;
         }
         else
         {
-            _dvdcss_debug( dvdcss, i_ret ? "disc is scrambled"
-                                         : "disc is unscrambled" );
+            print_debug( dvdcss, i_ret ? "disc is scrambled"
+                                       : "disc is unscrambled" );
             dvdcss->b_scrambled = i_ret;
         }
     }
@@ -287,26 +371,41 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
 
         if( i_ret < 0 )
         {
-            _dvdcss_close( dvdcss );
-            free( dvdcss->psz_device );
-            free( dvdcss );
-            return NULL;
+            print_debug( dvdcss, "could not get disc key" );
         }
     }
-
-#ifndef WIN32
-    if( psz_raw_device != NULL )
+    else
     {
-        _dvdcss_raw_open( dvdcss, psz_raw_device );
+        memset( dvdcss->css.p_disc_key, 0, KEY_SIZE );
+    }
+
+    /* If the cache is enabled, write the cache directory tag */
+    if( psz_cache )
+    {
+        char *psz_tag = "Signature: 8a477f597d28d172789f06886806bc55\r\n"
+            "# This file is a cache directory tag created by libdvdcss.\r\n"
+            "# For information about cache directory tags, see:\r\n"
+            "#   http://www.brynosaurus.com/cachedir/\r\n";
+        char psz_tagfile[PATH_MAX + 1 + 12 + 1];
+        int i_fd;
+
+        sprintf( psz_tagfile, "%s/CACHEDIR.TAG", psz_cache );
+        i_fd = open( psz_tagfile, O_RDWR|O_CREAT, 0644 );
+        if( i_fd >= 0 )
+        {
+            write( i_fd, psz_tag, strlen(psz_tag) );
+            close( i_fd );
+        }
     }
-#endif
 
     /* If the cache is enabled, extract a unique disc ID */
     if( psz_cache )
     {
         uint8_t p_sector[DVDCSS_BLOCK_SIZE];
-        unsigned char   psz_debug[PATH_MAX+30];
-        unsigned char * psz_data;
+        char psz_debug[PATH_MAX + 30];
+        char psz_key[1 + KEY_SIZE * 2 + 1];
+        char *psz_title;
+        uint8_t *psz_serial;
         int i;
 
         /* We read sector 0. If it starts with 0x000001ba (BE), we are
@@ -347,43 +446,58 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
         }
 
         /* Get the disc title */
-        psz_data = p_sector + 40;
-        psz_data[32] = '\0';
+        psz_title = (char *)p_sector + 40;
+        psz_title[32] = '\0';
 
         for( i = 0 ; i < 32 ; i++ )
         {
-            if( psz_data[i] <= ' ' )
+            if( psz_title[i] <= ' ' )
             {
-                psz_data[i] = '\0';
+                psz_title[i] = '\0';
                 break;
             }
-            else if( psz_data[i] == '/' || psz_data[i] == '\\' )
+            else if( psz_title[i] == '/' || psz_title[i] == '\\' )
             {
-                psz_data[i] = '-';
+                psz_title[i] = '-';
             }
         }
 
-        /* If it's not long enough, try the date + serial */
-        if( strlen( psz_data ) < 6 )
-        {
-            psz_data = p_sector + 813;
-            psz_data[16] = '\0';
+        /* Get the date + serial */
+        psz_serial = p_sector + 813;
+        psz_serial[16] = '\0';
 
-            /* Check that all characters are digits, otherwise convert. */
-            for( i = 0 ; i < 16 ; i++ )
+        /* Check that all characters are digits, otherwise convert. */
+        for( i = 0 ; i < 16 ; i++ )
+        {
+            if( psz_serial[i] < '0' || psz_serial[i] > '9' )
             {
-                if( psz_data[i] < '0' || psz_data[i] > '9' )
-                {
-                    sprintf( psz_data,
-                             "%.2X%.2X%.2X%.2X%.2X%.2X%.2X%.2X",
-                             psz_data[0], psz_data[1], psz_data[2],
-                             psz_data[3], psz_data[4], psz_data[5],
-                             psz_data[6], psz_data[7] );
-                    break;
-                }
+                char psz_tmp[16 + 1];
+                sprintf( psz_tmp,
+                         "%.2x%.2x%.2x%.2x%.2x%.2x%.2x%.2x",
+                         psz_serial[0], psz_serial[1], psz_serial[2],
+                         psz_serial[3], psz_serial[4], psz_serial[5],
+                         psz_serial[6], psz_serial[7] );
+                memcpy( psz_serial, psz_tmp, 16 );
+                break;
             }
         }
 
+        /* Get disk key, since some discs have got same title, manufacturing
+         * date and serial number, but different keys */
+        if( dvdcss->b_scrambled )
+        {
+             psz_key[0] = '-';
+             for( i = 0; i < KEY_SIZE; i++ )
+             {
+                 sprintf( &psz_key[1+i*2], "%.2x", dvdcss->css.p_disc_key[i] );
+             }
+             psz_key[1 + KEY_SIZE * 2] = '\0';
+        }
+        else
+        {
+             psz_key[0] = 0;
+        }
+
         /* We have a disc name or ID, we can create the cache dir */
         i = sprintf( dvdcss->psz_cachefile, "%s", psz_cache );
 #if !defined( WIN32 ) || defined( SYS_CYGWIN )
@@ -393,12 +507,13 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
 #endif
         if( i_ret < 0 && errno != EEXIST )
         {
-            _dvdcss_error( dvdcss, "failed creating cache directory" );
+            print_error( dvdcss, "failed creating cache directory" );
             dvdcss->psz_cachefile[0] = '\0';
             goto nocache;
         }
 
-        i += sprintf( dvdcss->psz_cachefile + i, "/%s/", psz_data );
+        i += sprintf( dvdcss->psz_cachefile + i, "/%s-%s%s", psz_title,
+                      psz_serial, psz_key );
 #if !defined( WIN32 ) || defined( SYS_CYGWIN )
         i_ret = mkdir( dvdcss->psz_cachefile, 0755 );
 #else
@@ -406,20 +521,28 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
 #endif
         if( i_ret < 0 && errno != EEXIST )
         {
-            _dvdcss_error( dvdcss, "failed creating cache subdirectory" );
+            print_error( dvdcss, "failed creating cache subdirectory" );
             dvdcss->psz_cachefile[0] = '\0';
             goto nocache;
         }
+        i += sprintf( dvdcss->psz_cachefile + i, "/");
 
         /* Pointer to the filename we will use. */
         dvdcss->psz_block = dvdcss->psz_cachefile + i;
 
         sprintf( psz_debug, "using CSS key cache dir: %s",
                             dvdcss->psz_cachefile );
-        _dvdcss_debug( dvdcss, psz_debug );
+        print_debug( dvdcss, psz_debug );
     }
     nocache:
 
+#ifndef WIN32
+    if( psz_raw_device != NULL )
+    {
+        _dvdcss_raw_open( dvdcss, psz_raw_device );
+    }
+#endif
+
     /* Seek at the beginning, just for safety. */
     dvdcss->pf_seek( dvdcss, 0 );
 
@@ -427,17 +550,17 @@ extern dvdcss_t dvdcss_open ( char *psz_target )
 }
 
 /**
- * \brief Return a string containing the latest error that occured in the
+ * \brief Return a string containing the latest error that occurred in the
  *        given \e libdvdcss instance.
  *
  * \param dvdcss a \e libdvdcss instance.
  * \return a null-terminated string containing the latest error message.
  *
  * This function returns a constant string containing the latest error that
- * occured in \e libdvdcss. It can be used to format error messages at your
+ * occurred in \e libdvdcss. It can be used to format error messages at your
  * convenience in your application.
  */
-extern char * dvdcss_error ( dvdcss_t dvdcss )
+LIBDVDCSS_EXPORT char * dvdcss_error ( dvdcss_t dvdcss )
 {
     return dvdcss->psz_error;
 }
@@ -466,7 +589,7 @@ extern char * dvdcss_error ( dvdcss_t dvdcss )
  * deprecated dvdcss_title() call. This flag is typically used when seeking
  * in a new title.
  */
-extern int dvdcss_seek ( dvdcss_t dvdcss, int i_blocks, int i_flags )
+LIBDVDCSS_EXPORT int dvdcss_seek ( dvdcss_t dvdcss, int i_blocks, int i_flags )
 {
     /* title cracking method is too slow to be used at each seek */
     if( ( ( i_flags & DVDCSS_SEEK_MPEG )
@@ -506,7 +629,7 @@ extern int dvdcss_seek ( dvdcss_t dvdcss, int i_blocks, int i_flags )
  * \warning dvdcss_read() expects to be able to write \p i_blocks *
  *          #DVDCSS_BLOCK_SIZE bytes in \p p_buffer.
  */
-extern int dvdcss_read ( dvdcss_t dvdcss, void *p_buffer,
+LIBDVDCSS_EXPORT int dvdcss_read ( dvdcss_t dvdcss, void *p_buffer,
                                           int i_blocks,
                                           int i_flags )
 {
@@ -529,7 +652,7 @@ extern int dvdcss_read ( dvdcss_t dvdcss, void *p_buffer,
         {
             if( ((uint8_t*)p_buffer)[0x14] & 0x30 )
             {
-                _dvdcss_error( dvdcss, "no key but found encrypted block" );
+                print_error( dvdcss, "no key but found encrypted block" );
                 /* Only return the initial range of unscrambled blocks? */
                 /* or fail completely? return 0; */
                 break;
@@ -579,7 +702,7 @@ extern int dvdcss_read ( dvdcss_t dvdcss, void *p_buffer,
  *          Moreover, all iov_len members of the iovec structures should be
  *          multiples of #DVDCSS_BLOCK_SIZE.
  */
-extern int dvdcss_readv ( dvdcss_t dvdcss, void *p_iovec,
+LIBDVDCSS_EXPORT int dvdcss_readv ( dvdcss_t dvdcss, void *p_iovec,
                                            int i_blocks,
                                            int i_flags )
 {
@@ -637,7 +760,7 @@ extern int dvdcss_readv ( dvdcss_t dvdcss, void *p_iovec,
  * by \e libdvdcss. On return, the #dvdcss_t is invalidated and may not be
  * used again.
  */
-extern int dvdcss_close ( dvdcss_t dvdcss )
+LIBDVDCSS_EXPORT int dvdcss_close ( dvdcss_t dvdcss )
 {
     dvd_title_t *p_title;
     int i_ret;
@@ -668,8 +791,21 @@ extern int dvdcss_close ( dvdcss_t dvdcss )
  *  Deprecated. See dvdcss_seek().
  */
 #undef dvdcss_title
-extern int dvdcss_title ( dvdcss_t dvdcss, int i_block )
+LIBDVDCSS_EXPORT int dvdcss_title ( dvdcss_t dvdcss, int i_block )
 {
     return _dvdcss_title( dvdcss, i_block );
 }
 
+/**
+ * \brief Return 1 if the DVD is scrambled, 0 otherwise.
+ *
+ * \param dvdcss a \e libdvdcss instance.
+ * \return 1 if the DVD is scrambled, 0 otherwise.
+ *
+ * This function returns whether the DVD is scrambled.
+ */
+LIBDVDCSS_EXPORT int dvdcss_is_scrambled ( dvdcss_t dvdcss )
+{
+    return dvdcss->b_scrambled;
+}
+
